package net.jieyuu.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import net.jieyuu.config.RabbitMQConfig;
import net.jieyuu.enums.BizCodeEnum;
import net.jieyuu.enums.CouponStateEnum;
import net.jieyuu.enums.ProductOrderStateEnum;
import net.jieyuu.enums.StockTaskStateEnum;
import net.jieyuu.exception.BizException;
import net.jieyuu.feign.ProductOrderFeignService;
import net.jieyuu.mapper.ProductTaskMapper;
import net.jieyuu.model.ProductDO;
import net.jieyuu.mapper.ProductMapper;
import net.jieyuu.model.ProductMessage;
import net.jieyuu.model.ProductTaskDO;
import net.jieyuu.request.LockProductRequest;
import net.jieyuu.request.OrderItemRequest;
import net.jieyuu.service.ProductService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import net.jieyuu.utils.JsonData;
import net.jieyuu.vo.ProductVO;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author jieyuu
 * @since 2024-08-07
 */
@Service
@Slf4j
public class ProductServiceImpl extends ServiceImpl<ProductMapper, ProductDO> implements ProductService {

    @Autowired
    private ProductMapper productMapper;

    @Autowired
    private ProductTaskMapper productTaskMapper;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Autowired
    private RabbitMQConfig rabbitMQConfig;

    @Autowired
    private ProductOrderFeignService productOrderFeignService;

    /**
     * 分页查询商品列表
     *
     * @param page
     * @param size
     * @return
     */
    @Override
    public Map<String, Object> page(int page, int size) {
        Page<ProductDO> pageInfo = new Page<>(page, size);
        IPage<ProductDO> productDOIPage = productMapper.selectPage(pageInfo, null);

        Map<String, Object> pageMap = new HashMap<>(3);

        pageMap.put("total_record", productDOIPage.getTotal());
        pageMap.put("total_page", productDOIPage.getPages());
        pageMap.put("current_data", productDOIPage.getRecords().stream().map(obj -> beanProcess(obj)).collect(Collectors.toList()));

        return pageMap;
    }

    /**
     * 根据id找商品详情
     *
     * @param productId
     * @return
     */
    @Override
    public ProductVO findDetailById(long productId) {
        ProductDO productDO = productMapper.selectById(productId);

        return beanProcess(productDO);
    }

    /**
     * 批量查询
     *
     * @param productIdList
     * @return
     */
    @Override
    public List<ProductVO> findProductByIdBatch(List<Long> productIdList) {
        List<ProductDO> productDOList = productMapper.selectList(new QueryWrapper<ProductDO>().in("id", productIdList));
        List<ProductVO> productVOList = productDOList.stream().map(obj -> beanProcess(obj)).collect(Collectors.toList());
        return productVOList;
    }

    /**
     * 锁定商品库存
     * 1)遍历商品，锁定每个商品购买数量
     * 2)每一次锁定的时候，都要发送延迟消息
     *
     * @param lockProductRequest
     * @return
     */
    @Override
    public JsonData lockProductStock(LockProductRequest lockProductRequest) {

        String outTradeNo = lockProductRequest.getOrderOutTradeNo();
        List<OrderItemRequest> orderItemList = lockProductRequest.getOrderItemList();

        // 批量查询itemList的商品情况，减少连接
        // 提取itemList里面的所有id
        List<Long> productIdList = orderItemList.stream().map(OrderItemRequest::getProductId).collect(Collectors.toList());

        //一次查询，结果分组
        //批量查询商品最新数据
        List<ProductVO> productVOList = this.findProductByIdBatch(productIdList);
        //根据id进行分组
        Map<Long, ProductVO> productMaps = productVOList.stream().collect(Collectors.toMap(ProductVO::getId, Function.identity()));

        for (OrderItemRequest item : orderItemList) {

            // 锁定商品记录
            int rows = productMapper.lockProductStock(item.getProductId(), item.getBuyNum());

            if (rows != 1) {
                throw new BizException(BizCodeEnum.ORDER_CONFIRM_LOCK_PRODUCT_FAIL);
            } else {
                // 插入product_task
                ProductVO productVO = productMaps.get(item.getProductId());
                ProductTaskDO productTaskDO = new ProductTaskDO();
                productTaskDO.setBuyNum(item.getBuyNum());
                productTaskDO.setLockState(StockTaskStateEnum.LOCK.name());
                productTaskDO.setProductId(item.getProductId());
                productTaskDO.setOutTradeNo(outTradeNo);

                // 假若不需要插入商品名，那么前文批量查询就不需要了
                // 此处为冗余字段
                productTaskDO.setProductName(productVO.getTitle());

                // 插入product_task
                productTaskMapper.insert(productTaskDO);
                log.info("商品库存锁定-插入商品product_task成功:{}", productTaskDO);

                // 发送MQ延迟消息, 延迟解锁商品库存
                ProductMessage productMessage = new ProductMessage();
                productMessage.setOutTradeNo(outTradeNo);
                productMessage.setTaskId(productTaskDO.getId());

                rabbitTemplate.convertAndSend(rabbitMQConfig.getEventExchange(), rabbitMQConfig.getStockReleaseDelayRoutingKey(), productMessage);
                log.info("商品库存锁定信息延迟消息发送成功:{}", productMessage);
            }
        }
        return null;
    }

    /**
     * 释放商品库存
     *
     * @param productMessage
     * @return
     */
    @Override
    public boolean releaseProductStock(ProductMessage productMessage) {
        // 查询商品任务
        ProductTaskDO taskDO = productTaskMapper.selectOne(new QueryWrapper<ProductTaskDO>().eq("id", productMessage.getTaskId()));
        if (taskDO == null) {
            log.warn("工作单不存在,消息体为:{}", productMessage);
        }

        // lock状态才处理
        if (StockTaskStateEnum.LOCK.name().equalsIgnoreCase(taskDO.getLockState())) {

            // 调用feign查询对应订单的状态
            // 锁定状态才处理
            JsonData jsonData = productOrderFeignService.queryProductOrderState(productMessage.getOutTradeNo());
            // 正常响应
            if (jsonData.getCode() == 0) {
                // 判断状态
                String state = jsonData.getData().toString();
                if (ProductOrderStateEnum.NEW.name().equalsIgnoreCase(state)) {
                    // 状态是NEW，返回给消息队列，重新投递
                    log.warn("状态是NEW,返回给消息队列,重新投递:{}", productMessage);
                    return false;
                }

                // 订单已支付
                if (ProductOrderStateEnum.PAY.name().equalsIgnoreCase(state)) {
                    // 已经支付，修改task状态为finish
                    taskDO.setLockState(StockTaskStateEnum.FINISH.name());
                    productTaskMapper.update(taskDO, new QueryWrapper<ProductTaskDO>().eq("id", productMessage.getTaskId()));
                    log.info("订单已经支付,修改库存锁定工单为FINISH状态:{}", productMessage);
                    return true;
                }

            }

            log.warn("订单不存在,或订单取消,确认消息,修改task状态为CANCLE,恢复商品库存,message:{}", productMessage);
            taskDO.setLockState(StockTaskStateEnum.CANCEL.name());
            // 修改task状态为CANCEL
            productTaskMapper.update(taskDO, new QueryWrapper<ProductTaskDO>().eq("id", productMessage.getTaskId()));
            // 恢复商品库存
            // 即锁定库存的值减去当前的buyNum
            productMapper.unlockProductStock(taskDO.getProductId(),taskDO.getBuyNum());
            return true;

        } else {
            // 工单状态不是lock
            // 认定为可能为重复消费,不需要重新投递
            log.warn("工单状态不为LOCK,state={},消息体:{}", taskDO.getLockState(), productMessage);
            return true;
        }

    }

    private ProductVO beanProcess(ProductDO obj) {
        ProductVO productVO = new ProductVO();
        BeanUtils.copyProperties(obj, productVO);

        return productVO;
    }
}
